#!/usr/bin/env python
# -*- coding: utf-8 -*-

#===============================================================================
# 
#
#   THIS FILE IS AUTOGENERATED DO NOT MODIFY
#
#
#===============================================================================

# Copyright 2012 Liricus SRL <info@liricus.com.ar>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Fra


#===============================================================================
# IMPORTS
#===============================================================================

import unittest
import os
import json
import random
import hashlib
import tempfile
import datetime
import time

import peewee

from flaskext import uploads


#===============================================================================
# IMPORTS AND FUNCTIONS TO CONFIGURE SOCIALFRAME SERVER
#===============================================================================


import settings
FD, DB = tempfile.mkstemp("_socialframe_test.db")
settings.HOST = "localhost"
settings.PORT = 8080
settings.DB = {
    "schema": "sqlite3",
    "name": DB,
}
settings.IMAGES = tempfile.mkdtemp()
settings.TESTING = True
settings.USER_NEVER_ACTIVE_EXPIRATION = 0
settings.USER_UNLOGED_EXPIRATION = 0
settings.USER_SUSPENDED_EXPIRATION = 0

import socialframe_server
socialframe_server.init_settings(settings)

from socialframe_server import models, core, jobs
from socialframe_server.utils import fileutils

import test_data


#===============================================================================
# CONSTANTS
#===============================================================================

PATH = os.path.abspath(os.path.dirname(__file__))

TEST_APP = core.APP.test_client()


#===============================================================================
# FUNCTIONS
#===============================================================================

def mutter(f):
    return None


#===============================================================================
# BASE TESTS
#===============================================================================

class SocialFrameBaseTest(unittest.TestCase):
    
    def full_url(self, action, args):
        return "/" + "/".join([action] + list(args))
    
    def giveme_user(self, password=None, status=models.User.STATUS_ACTIVE):
        user = models.User()
        user.username = self.email
        user.email = self.email
        user.birth = datetime.datetime.now().date()
        user.sex = "m" if random.randint(0, 1) else "f"
        user.fname = str(random.random())
        user.lname = str(random.random())
        user.last_login = datetime.datetime.now()
        user.status = status
        user.set_password(password or str(random.random()))
        user.save()
        return user
    
    def call(self, action, args=(), data={}, error={}, method="GET"):
        url = self.full_url(action, args)
        if method == "GET":
            method = TEST_APP.get
        elif method == "POST":
            method = TEST_APP.post
        response = method(url, data=data)
        data = json.loads(response.data)
        if data["action"] == "index":
            self.assertEquals("/", url)
        else:
            self.assertIn(data["action"], [action, None])
        msg = u"url {} - data {}".format(url, str(data))
        if error:
            self.assertEqual(data["error"], True)
            for k, v in error.items():
                self.assertEqual(data[k], v, msg=msg)
        else:
            self.assertNotEqual(data["error"], True, msg=msg)
        #time.sleep(0.5)
        return data
    
    def setUp(self):
        self._unique = random.random() + time.time()
        self.email = "test@socialframe.com"
    
    def unique_name(self):
        self._unique += 1
        return str(self._unique)
    
    def tearDown(self):
        try:
            self.call("logout")
        except:
            pass
        for model in models.autocreatetables():
            for instance in model.select():
                instance.delete_instance(recursive=True)
    
#===============================================================================
# TEST
#===============================================================================

class APITest(SocialFrameBaseTest):
    
    def test_index(self):
        self.call("")

    def test_api_help(self):
        self.call("apihelp")
    
    def test_create_user(self):
        password = str(random.random())
        fname = str(random.random())
        lname = str(random.random())
        sex = "m" if random.randint(0, 1) else "f"
        birth = datetime.datetime.now().strftime("%Y%m%d")
        
        self.call(
            "register_user", (self.email, fname, lname, birth, sex, password)
        )
        user = models.User.get(email=self.email)
        self.assertFalse(user.active)
        profile = models.Profile.get(user=user)
        self.assertEquals(profile.operation, models.Profile.OPERATION_CREATE_USER)
        self.call(
            "confirm_registration", (self.email, profile.operation_hash)
        )
        user = models.User.get(email=self.email)
        self.assertTrue(user.active)

    def test_login_logout(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login", (user.email, password))
        data = self.call("is_loggedin")
        self.assertTrue(data["is_loggedin"])
        self.call("logout")

    def test_reset_password(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.assertTrue(user.check_password(password))
        self.call("login", (self.email, password))
        self.call("reset_password", (self.email,))
        user = models.User.get(email=self.email)
        self.assertFalse(user.check_password(password))
        self.call("login", (self.email, password), error={"code": 403})

    def test_delete_user(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login", (self.email, password))
        self.call("delete_user")
        user = models.User.get(email=self.email)
        profile = models.Profile.get(user=user)
        self.assertEquals(profile.operation, models.Profile.OPERATION_DELETE_USER)
        self.call("confirm_delete", (self.email, profile.operation_hash))
        with self.assertRaises(peewee.DoesNotExist):
            user = models.User.get(email=self.email)
            user.profile

    def test_crud_bubles(self):
        password = str(random.random())
        user = self.giveme_user(password)
        for listed in (True, False):
            for bubble_password in (True, False):
                for tags in (True, False):
                    for desc in (True, False):
                        bubble_name = str(random.random())
    
                        if listed:
                            listed = "true"
                        else:
                            listed = "false"
                            
                        if desc:
                            desc = str(random.random())
                        else:
                            desc = "null"
    
                        if bubble_password:
                            bubble_password = str(random.random())
                        else:
                            bubble_password = "null"
    
                        if tags:
                            tags = ",".join([
                                hashlib.sha1(str(random.random())).hexdigest()
                                for c in range(random.randint(1, 10))
                            ])
                        else:
                            tags = "null"
                        args = (bubble_name, listed, desc, tags, bubble_password)
                        self.call(
                            "create_bubble", args, error={"code": 403}
                        )
    
                        self.call("login", (self.email, password))
                        self.call("create_bubble", args)
                        data = self.call("list_bubbles", (user.email,))
    
                        if listed == "true":
                            self.assertIn(bubble_name, data["bubbles"])
                            bubble_data = data["bubbles"][bubble_name]
                            if tags != "null":
                                for tag in tags.split(","):
                                    self.assertIn(tag.strip(), bubble_data["tags"])
                            else:
                                self.assertFalse(bubble_data["tags"])
                        else:
                            self.assertNotIn(bubble_name, data["bubbles"])
    
                        data = self.call("my_bubbles")
    
                        self.assertIn(bubble_name, data["my_bubbles"])
                        bubble_data = data["my_bubbles"][bubble_name]
                        if tags != "null":
                            for tag in tags.split(","):
                                self.assertIn(tag.strip(), bubble_data["tags"])
                        else:
                            self.assertFalse(bubble_data["tags"])
    
                        if bubble_password != "null":
                            self.call(
                                "list_photos", (user.email, bubble_name, "null"),
                                error={"code": 403}
                            )
    
                        self.call("delete_bubble", (bubble_name,))
    
                        data = self.call("list_bubbles", (user.email,))
                        self.assertNotIn(bubble_name, data["bubbles"])
    
                        self.call(
                            "listphotos", (user.email, bubble_name),
                            error={"code": 404}
                        )
    
                        self.call("logout")
    
    def test_crud_image(self):
        password = str(random.random())
        user = self.giveme_user(password)
        
        photos = list(test_data.ls())
        hashes = {}
        for photo in photos:
            #upload_name = uploads.secure_filename(photo)
            hashed_name = ".".join([
                hashlib.sha1(photo).hexdigest(), photo.rsplit(".", 1)[1]
            ])
            hashes[hashed_name] = {
                "sha": fileutils.sha(photo)
            }
        bubbles = []
        with core.APP_DB.database.transaction():
            for bubble_password in (True, False):
                bubble = models.Bubble()
                bubble.user = user
                bubble.name = self.unique_name()
                bubble.listed = bool(random.randint(0, 1))
                if bubble_password:
                    bubble_password = str(random.random())
                    bubble.set_password(bubble_password)
                else:
                    bubble_password = "null"
                bubble.save()
                bubbles.append((bubble, bubble_password))
            
        for bubble, bubble_password in bubbles:
            for photo in test_data.ls():
                self.call(
                    "upload_photo", (bubble.name,),
                    data={"photo": open(photo)},
                    method="POST", error={"code": 403}
                )
            
            data = self.call(
                "list_photos", (user.email, bubble.name, bubble_password)
            )
            self.assertFalse(data["photos"])
            
            self.call("login", (self.email, password))
            
            for photo in photos:
                self.call(
                    "upload_photo", (bubble.name,),
                    data={"photo": open(photo)},
                    method="POST"
                )

            data = self.call(
                "list_photos", (user.email, bubble.name, bubble_password)
            )
            self.assertEquals(len(data["photos"]), len(photos))
            
            for pname, pdata in data["photos"].items():
                for size in settings.IMAGE_SIZES.keys():
                    url = self.full_url(
                        "get_photo", (self.email, bubble.name, pname, size, bubble_password)
                    )
                    response = TEST_APP.get(url)
                    self.assertTrue(response.mimetype.startswith("image/"))
                    
                
            for pname in data["photos"].keys():
                data = self.call("del_photo", (bubble.name, pname))
                subdata = self.call(
                    "list_photos", (user.email, bubble.name, bubble_password)
                )
                self.assertNotIn(pname, subdata["photos"])

            self.call("logout")

    def test_giveme_bubbles(self):
        password = str(random.random())
        user = self.giveme_user(password)
        tags = [
            hashlib.sha1(str(random.random())).hexdigest()
            for c in range(100)
        ]
        with core.APP_DB.database.transaction():
            for bubble_password in (True, False):
                for bubble_listed in (True, False):
                    bubble = models.Bubble()
                    bubble.user = user
                    bubble.name = self.unique_name()
                    bubble.listed = bubble_listed
                    if bubble_password:
                        bubble_password = str(random.random())
                        bubble.set_password(bubble_password)
                    else:
                        bubble_password = "null"
                    bubble.save()
                    if random.randint(0, 100) < 80:
                        selected_tags = set()
                        for idx in range(len(tags)):
                            selected_tags.add(random.choice(tags))
                        bubble.set_tags(selected_tags)
        data = self.call("give_me_bubbles")
        for bdata in data["bubbles"].values():
            self.assertTrue(bdata["listed"])

    def test_find(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("find", ("users", user.email))
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        self.call("find", ("bubbles", bubble.name))

    def test_sizes(self):
        sizes = sorted(self.call("sizes")["sizes"].items())
        self.assertEquals(sizes, sorted(settings.IMAGE_SIZES.items()))

    def test_change_user(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login", (self.email, password))
        values = {}
        for k in ("fname", "lname", "password"):
            new_value = hashlib.sha512(str(random.random())).hexdigest()
            values[k] = new_value
        to_url = ",".join((k + ":" + v) for k, v in values.items())
        self.call("change_user", (to_url,))
        user = models.User.get(email=self.email)
        for k in ("fname", "lname", "password"):
            if k == "password":
                user.check_password(values[k])
            else:
                self.assertEquals(getattr(user, k), values[k])
    
    def test_change_bubbles(self):
        password = str(random.random())
        user = self.giveme_user(password)
        bubbles = []
        with core.APP_DB.database.transaction():
            for bubble_password in (True, False):
                bubble = models.Bubble()
                bubble.user = user
                bubble.listed = bool(random.randint(0, 1))
                bubble.name = self.unique_name()
                if bubble_password:
                    bubble_password = str(random.random())
                    bubble.set_password(bubble_password)
                else:
                    bubble_password = "null"
                bubble.save()
                bubbles.append(bubble.name)
        self.call("login", (self.email, password))
        names = []
        passwords = []
        for bname in bubbles:
            values = {}
            for k in ("name", "password"):
                new_value = hashlib.sha512(str(random.random())).hexdigest()
                values[k] = new_value
                if k == "name":
                    names.append(new_value)
                elif k == "password":
                    passwords.append(new_value)
            to_url = ",".join((k + ":" + v) for k, v in values.items())
            self.call("change_bubble", (bname, to_url))
        for b in models.Bubble.select():
            self.assertIn(b.name, names)
            self.assertTrue(any(map(b.check_password, passwords)))

    def test_reactivate_user(self):
        password = str(random.random())
        user = self.giveme_user(password, status=models.User.STATUS_SUSPENDED)
        self.assertFalse(user.active)
        profile = models.Profile.get(user=user)
        profile.operation = models.Profile.OPERATION_REACTIVATE_USER
        profile.save()
        self.call("reactivate_user", (self.email, profile.operation_hash))
        user = models.User.get(id=user.id)
        self.assertTrue(user.active)
        
    def test_validate_bubble_password(self):
        user = self.giveme_user()
        bubbles = []
        with core.APP_DB.database.transaction():
            for bubble_password in (True, False):
                bubble = models.Bubble()
                bubble.user = user
                bubble.name = self.unique_name()
                bubble.listed = bool(random.randint(0, 1))
                if bubble_password:
                    bubble_password = str(random.random())
                    bubble.set_password(bubble_password)
                else:
                    bubble_password = "null"
                bubble.save()
                bubbles.append((bubble, bubble_password))
        for bubble, bubble_password in bubbles:
            args = (self.email, bubble.name, bubble_password)
            vp = self.call("validate_bubble_password", args)["valid_password"]
            self.assertTrue(vp)
            args = (self.email, bubble.name, self.unique_name())
            vp = self.call("validate_bubble_password", args)["valid_password"]
            self.assertFalse(vp)
        

#===============================================================================
# CRON TEST
#===============================================================================

class JobsTest(SocialFrameBaseTest):
    
    def test_delete_inactive_users(self):
        user = self.giveme_user(status=models.User.STATUS_INACTIVE)
        self.assertEquals(models.User.filter(id=user.id).count(), 1)
        jobs.delete_inactive_users()
        self.assertEquals(models.User.filter(id=user.id).count(), 0)
        user = self.giveme_user()
        self.assertEquals(models.User.filter(id=user.id).count(), 1)
        jobs.delete_inactive_users()
        self.assertEquals(models.User.filter(id=user.id).count(), 1)
        
    def test_suspend_user(self):
        user = self.giveme_user(status=models.User.STATUS_ACTIVE)
        query = models.User.filter(id=user.id,
                                   status=models.User.STATUS_SUSPENDED)
        self.assertEquals(query.count(), 0)
        jobs.suspend_user()
        self.assertEquals(query.count(), 1)
        
    def test_delete_suspended_user(self):
        user = self.giveme_user(status=models.User.STATUS_SUSPENDED)
        query = models.User.filter(id=user.id,
                                   status=models.User.STATUS_SUSPENDED)
        self.assertEquals(query.count(), 1)
        jobs.delete_suspended_user()
        self.assertEquals(query.count(), 0)
        user = self.giveme_user()
        query = models.User.filter(id=user.id)
        self.assertEquals(query.count(), 1)
        jobs.delete_suspended_user()
        self.assertEquals(query.count(), 1)

#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    unittest.main(verbosity=2)
